@@ -32,11 +32,12 @@
 #include "encryption/Encrypter.h"
 #include "impl/UnknownElement.h"
 #include "io/HTTPResponse.h"
-#include "security/TrustEngine.h"
-#include "security/OpenSSLCryptoX509CRL.h"
 #include "security/CredentialResolver.h"
+#include "security/DataSealer.h"
 #include "security/KeyInfoResolver.h"
+#include "security/OpenSSLCryptoX509CRL.h"
 #include "security/PathValidator.h"
+#include "security/TrustEngine.h"
 #include "signature/KeyInfo.h"
 #include "signature/Signature.h"
 #include "soap/SOAP.h"
@@ -83,13 +84,6 @@ using namespace xercesc;
 using namespace boost;
 using namespace std;
 
-#ifdef WIN32
-# if (OPENSSL_VERSION_NUMBER >= 0x00908000)
-#  define XMLTOOLING_OPENSSL_HAVE_SHA2 1
-# endif
-#endif
-
-
 DECL_XMLTOOLING_EXCEPTION_FACTORY(XMLParserException,xmltooling);
 DECL_XMLTOOLING_EXCEPTION_FACTORY(XMLObjectException,xmltooling);
 DECL_XMLTOOLING_EXCEPTION_FACTORY(MarshallingException,xmltooling);
@@ -111,6 +105,15 @@ using namespace xmlsignature;
 namespace {
     static XMLToolingInternalConfig g_config;
 #ifndef XMLTOOLING_NO_XMLSEC
+// NOTE:
+// "The old locking functions have been removed completely without compatibility macros"
+// see:
+// https://www.openssl.org/docs/manmaster/crypto/CRYPTO_THREAD_lock_free.html
+//
+// For now we just make the callback compile.  More work TBD
+#ifndef CRYPTO_LOCK
+#define CRYPTO_LOCK 1
+#endif
     static ptr_vector<Mutex> g_openssl_locks;
 
     extern "C" void openssl_locking_callback(int mode,int n,const char *file,int line)
@@ -128,7 +131,6 @@ namespace {
     }
 # endif
 
-# ifdef XMLTOOLING_XMLSEC_DEBUGLOGGING
     class TXFMOutputLog : public TXFMBase {
 	    TXFMOutputLog();
     public:
@@ -147,13 +149,13 @@ namespace {
             m_log.debug("\n----- BEGIN SIGNATURE DEBUG -----\n");
         }
 
-	    TXFMBase::ioType getInputType() {
+	    TXFMBase::ioType getInputType() const {
             return TXFMBase::BYTE_STREAM;
         }
-	    TXFMBase::ioType getOutputType() {
+	    TXFMBase::ioType getOutputType() const {
             return TXFMBase::BYTE_STREAM;
         }
-	    TXFMBase::nodeType getNodeType() {
+	    TXFMBase::nodeType getNodeType() const {
             return TXFMBase::DOM_NODE_NONE;
         }
 
@@ -162,16 +164,6 @@ namespace {
             m_log.debug(string(reinterpret_cast<char* const>(toFill), sz));
 	        return sz;
         }
-
-	    DOMDocument* getDocument() {
-            return nullptr;
-        }
-	    DOMNode* getFragmentNode() {
-            return nullptr;
-        }
-	    const XMLCh* getFragmentId() {
-            return nullptr;
-        }
 	
     private:
         Category& m_log;
@@ -182,7 +174,6 @@ namespace {
             return new TXFMOutputLog(doc);
         return nullptr;
     }
-# endif
 
 #endif
 
@@ -213,13 +204,7 @@ XMLToolingInternalConfig& XMLToolingInternalConfig::getInternalConfig()
     return g_config;
 }
 
-#ifndef XMLTOOLING_NO_XMLSEC
-XMLToolingConfig::XMLToolingConfig()
-    : m_keyInfoResolver(nullptr), m_replayCache(nullptr), m_pathResolver(nullptr), m_templateEngine(nullptr), m_urlEncoder(nullptr), clock_skew_secs(180)
-#else
-XMLToolingConfig::XMLToolingConfig()
-    : m_pathResolver(nullptr), m_templateEngine(nullptr), m_urlEncoder(nullptr), clock_skew_secs(180)
-#endif
+XMLToolingConfig::XMLToolingConfig() : clock_skew_secs(180)
 {
 }
 
@@ -230,65 +215,66 @@ XMLToolingConfig::~XMLToolingConfig()
 #ifndef XMLTOOLING_LITE
 const KeyInfoResolver* XMLToolingConfig::getKeyInfoResolver() const
 {
-    return m_keyInfoResolver;
+    return m_keyInfoResolver.get();
 }
 
 ReplayCache* XMLToolingConfig::getReplayCache() const
 {
-    return m_replayCache;
+    return m_replayCache.get();
+}
+
+const DataSealer* XMLToolingConfig::getDataSealer() const
+{
+    return m_dataSealer.get();
 }
 
 void XMLToolingConfig::setKeyInfoResolver(xmltooling::KeyInfoResolver *keyInfoResolver)
 {
-    delete m_keyInfoResolver;
-    m_keyInfoResolver = keyInfoResolver;
+    m_keyInfoResolver.reset(keyInfoResolver);
 }
 
 void XMLToolingConfig::setReplayCache(ReplayCache* replayCache)
 {
-    delete m_replayCache;
-    m_replayCache = replayCache;
+    m_replayCache.reset(replayCache);
+}
+
+void XMLToolingConfig::setDataSealer(DataSealer* dataSealer)
+{
+    m_dataSealer.reset(dataSealer);
 }
 #endif
 
 PathResolver* XMLToolingConfig::getPathResolver() const
 {
-    return m_pathResolver;
+    return m_pathResolver.get();
 }
 
 TemplateEngine* XMLToolingConfig::getTemplateEngine() const
 {
-    return m_templateEngine;
+    return m_templateEngine.get();
 }
 
 const URLEncoder* XMLToolingConfig::getURLEncoder() const
 {
-    return m_urlEncoder;
+    return m_urlEncoder.get();
 }
 
 void XMLToolingConfig::setPathResolver(PathResolver* pathResolver)
 {
-    delete m_pathResolver;
-    m_pathResolver = pathResolver;
+    m_pathResolver.reset(pathResolver);
 }
 
 void XMLToolingConfig::setTemplateEngine(TemplateEngine* templateEngine)
 {
-    delete m_templateEngine;
-    m_templateEngine = templateEngine;
+    m_templateEngine.reset(templateEngine);
 }
 
 void XMLToolingConfig::setURLEncoder(URLEncoder* urlEncoder)
 {
-    delete m_urlEncoder;
-    m_urlEncoder = urlEncoder;
+    m_urlEncoder.reset(urlEncoder);
 }
 
-XMLToolingInternalConfig::XMLToolingInternalConfig() :
-#ifndef XMLTOOLING_NO_XMLSEC
-    m_xsecProvider(nullptr),
-#endif
-    m_initCount(0), m_lock(Mutex::create()), m_parserPool(nullptr), m_validatingPool(nullptr)
+XMLToolingInternalConfig::XMLToolingInternalConfig() : m_initCount(0), m_lock(Mutex::create())
 {
 }
 
@@ -347,7 +333,7 @@ bool XMLToolingInternalConfig::log_config(const char* config)
         }
         else {
             string path(config);
-            PropertyConfigurator::configure(m_pathResolver ? m_pathResolver->resolve(path, PathResolver::XMLTOOLING_CFG_FILE) : path);
+            PropertyConfigurator::configure(m_pathResolver.get() ? m_pathResolver->resolve(path, PathResolver::XMLTOOLING_CFG_FILE) : path);
         }
 
 #ifndef XMLTOOLING_NO_XMLSEC
@@ -366,7 +352,7 @@ bool XMLToolingInternalConfig::log_config(const char* config)
     return true;
 }
 
-bool XMLToolingInternalConfig::init()
+bool XMLToolingInternalConfig::init(bool deprecationSupport)
 {
 #ifdef _DEBUG
     xmltooling::NDC ndc("init");
@@ -412,22 +398,16 @@ bool XMLToolingInternalConfig::init()
 
 #ifndef XMLTOOLING_NO_XMLSEC
         XSECPlatformUtils::Initialise();
-# ifdef XMLTOOLING_XMLSEC_DEBUGLOGGING
         XSECPlatformUtils::SetReferenceLoggingSink(TXFMOutputLogFactory);
-# endif
-        m_xsecProvider = new XSECProvider();
+        m_xsecProvider.reset(new XSECProvider());
         log.debug("XML-Security %s initialization complete", XSEC_FULLVERSIONDOT);
 #endif
 
-        m_parserPool = new ParserPool();
-        m_validatingPool = new ParserPool(true,true);
+        m_parserPool.reset(new ParserPool());
+        m_validatingPool.reset(new ParserPool(true,true));
 
-        m_pathResolver = new PathResolver();
-        m_urlEncoder = new URLEncoder();
-
-        // Load catalogs from deprecated path setting.
-        if (!catalog_path.empty())
-            m_validatingPool->loadCatalogs(catalog_path.c_str());
+        m_pathResolver.reset(new PathResolver());
+        m_urlEncoder.reset(new URLEncoder());
 
         // default registrations
         XMLObjectBuilder::registerDefaultBuilder(new UnknownElementBuilder());
@@ -448,6 +428,7 @@ bool XMLToolingInternalConfig::init()
         REGISTER_XMLTOOLING_EXCEPTION_FACTORY(XMLSecurityException,xmltooling);
         REGISTER_XMLTOOLING_EXCEPTION_FACTORY(SignatureException,xmlsignature);
         REGISTER_XMLTOOLING_EXCEPTION_FACTORY(EncryptionException,xmlencryption);
+        registerDataSealerKeyStrategies();
         registerKeyInfoClasses();
         registerEncryptionClasses();
         registerCredentialResolvers();
@@ -455,7 +436,7 @@ bool XMLToolingInternalConfig::init()
         registerPathValidators();
         registerTrustEngines();
         registerXMLAlgorithms();
-        m_keyInfoResolver = KeyInfoResolverManager.newPlugin(INLINE_KEYINFO_RESOLVER,nullptr);
+        m_keyInfoResolver.reset(KeyInfoResolverManager.newPlugin(INLINE_KEYINFO_RESOLVER,nullptr,deprecationSupport));
 #endif
 
 #ifndef XMLTOOLING_LITE
@@ -530,23 +511,17 @@ void XMLToolingInternalConfig::term()
     TrustEngineManager.deregisterFactories();
     CredentialResolverManager.deregisterFactories();
     KeyInfoResolverManager.deregisterFactories();
+    DataSealerKeyStrategyManager.deregisterFactories();
     m_algorithmMap.clear();
 
-    delete m_keyInfoResolver;
-    m_keyInfoResolver = nullptr;
-
-    delete m_replayCache;
-    m_replayCache = nullptr;
+    m_keyInfoResolver.reset();
+    m_replayCache.reset();
+    m_dataSealer.reset();
 #endif
 
-    delete m_pathResolver;
-    m_pathResolver = nullptr;
-
-    delete m_templateEngine;
-    m_templateEngine = nullptr;
-
-    delete m_urlEncoder;
-    m_urlEncoder = nullptr;
+    m_pathResolver.reset();
+    m_templateEngine.reset();
+    m_urlEncoder.reset();
 
     for (vector<void*>::reverse_iterator i=m_libhandles.rbegin(); i!=m_libhandles.rend(); i++) {
 #if defined(WIN32)
@@ -565,17 +540,14 @@ void XMLToolingInternalConfig::term()
     }
     m_libhandles.clear();
 
-    delete m_parserPool;
-    m_parserPool=nullptr;
-    delete m_validatingPool;
-    m_validatingPool=nullptr;
+    m_parserPool.reset();
+    m_validatingPool.reset();
 
     for_each(m_namedLocks.begin(), m_namedLocks.end(), cleanup_pair<string,Mutex>());
     m_namedLocks.clear();
 
 #ifndef XMLTOOLING_NO_XMLSEC
-    delete m_xsecProvider;
-    m_xsecProvider=nullptr;
+    m_xsecProvider.reset();
     XSECPlatformUtils::Terminate();
 #endif
 
@@ -585,6 +557,7 @@ void XMLToolingInternalConfig::term()
     curl_global_cleanup();
 #endif
    Category::getInstance(XMLTOOLING_LOGCAT ".Config").info("%s library shutdown complete", PACKAGE_STRING);
+   Category::shutdown();
 }
 
 Lockable* XMLToolingInternalConfig::lock()
@@ -757,51 +730,51 @@ void XMLToolingInternalConfig::registerXMLAlgorithms()
 
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURIMD5, nullptr, 0, ALGTYPE_DIGEST);
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURISHA1, nullptr, 0, ALGTYPE_DIGEST);
-#if defined(XMLTOOLING_OPENSSL_HAVE_SHA2) && !defined(OPENSSL_NO_SHA256)
+#if defined(XSEC_OPENSSL_HAVE_SHA2) && !defined(OPENSSL_NO_SHA256)
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURISHA224, nullptr, 0, ALGTYPE_DIGEST);
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURISHA256, nullptr, 0, ALGTYPE_DIGEST);
 #endif
-#if defined(XMLTOOLING_OPENSSL_HAVE_SHA2) && !defined(OPENSSL_NO_SHA512)
+#if defined(XSEC_OPENSSL_HAVE_SHA2) && !defined(OPENSSL_NO_SHA512)
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURISHA384, nullptr, 0, ALGTYPE_DIGEST);
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURISHA512, nullptr, 0, ALGTYPE_DIGEST);
 #endif
 
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURIDSA_SHA1, "DSA", 0, ALGTYPE_SIGN);
-#if defined(URI_ID_DSA_SHA256) && defined(XMLTOOLING_OPENSSL_HAVE_SHA2) && !defined(OPENSSL_NO_SHA256)
+#if defined(URI_ID_DSA_SHA256) && defined(XSEC_OPENSSL_HAVE_SHA2) && !defined(OPENSSL_NO_SHA256)
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURIDSA_SHA256, "DSA", 0, ALGTYPE_SIGN);
 #endif
 
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURIRSA_MD5, "RSA", 0, ALGTYPE_SIGN);
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURIRSA_SHA1, "RSA", 0, ALGTYPE_SIGN);
-#if defined(XMLTOOLING_OPENSSL_HAVE_SHA2) && !defined(OPENSSL_NO_SHA256)
+#if defined(XSEC_OPENSSL_HAVE_SHA2) && !defined(OPENSSL_NO_SHA256)
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURIRSA_SHA224, "RSA", 0, ALGTYPE_SIGN);
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURIRSA_SHA256, "RSA", 0, ALGTYPE_SIGN);
 #endif
-#if defined(XMLTOOLING_OPENSSL_HAVE_SHA2) && !defined(OPENSSL_NO_SHA512)
+#if defined(XSEC_OPENSSL_HAVE_SHA2) && !defined(OPENSSL_NO_SHA512)
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURIRSA_SHA384, "RSA", 0, ALGTYPE_SIGN);
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURIRSA_SHA512, "RSA", 0, ALGTYPE_SIGN);
 #endif
 
 #ifdef XSEC_OPENSSL_HAVE_EC
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURIECDSA_SHA1, "EC", 0, ALGTYPE_SIGN);
-# if defined(XMLTOOLING_OPENSSL_HAVE_SHA2) && !defined(OPENSSL_NO_SHA256)
+# if defined(XSEC_OPENSSL_HAVE_SHA2) && !defined(OPENSSL_NO_SHA256)
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURIECDSA_SHA256, "EC", 0, ALGTYPE_SIGN);
 #  ifdef URI_ID_ECDSA_SHA224
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURIECDSA_SHA224, "EC", 0, ALGTYPE_SIGN);
 #  endif
 # endif
-# if defined(XMLTOOLING_OPENSSL_HAVE_SHA2) && !defined(OPENSSL_NO_SHA512)
+# if defined(XSEC_OPENSSL_HAVE_SHA2) && !defined(OPENSSL_NO_SHA512)
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURIECDSA_SHA384, "EC", 0, ALGTYPE_SIGN);
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURIECDSA_SHA512, "EC", 0, ALGTYPE_SIGN);
 # endif
 #endif
 
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURIHMAC_SHA1, "HMAC", 0, ALGTYPE_SIGN);
-#if defined(XMLTOOLING_OPENSSL_HAVE_SHA2) && !defined(OPENSSL_NO_SHA256)
+#if defined(XSEC_OPENSSL_HAVE_SHA2) && !defined(OPENSSL_NO_SHA256)
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURIHMAC_SHA224, "HMAC", 0, ALGTYPE_SIGN);
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURIHMAC_SHA256, "HMAC", 0, ALGTYPE_SIGN);
 #endif
-#if defined(XMLTOOLING_OPENSSL_HAVE_SHA2) && !defined(OPENSSL_NO_SHA512)
+#if defined(XSEC_OPENSSL_HAVE_SHA2) && !defined(OPENSSL_NO_SHA512)
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURIHMAC_SHA384, "HMAC", 0, ALGTYPE_SIGN);
     registerXMLAlgorithm(DSIGConstants::s_unicodeStrURIHMAC_SHA512, "HMAC", 0, ALGTYPE_SIGN);
 #endif
